@@ -28,7 +28,8 @@ from .exceptions import (
 )
 from .frame_buffer import FrameBuffer
 from .settings import (
-    Settings, HEADER_TABLE_SIZE, INITIAL_WINDOW_SIZE, MAX_FRAME_SIZE
+    Settings, HEADER_TABLE_SIZE, INITIAL_WINDOW_SIZE, MAX_FRAME_SIZE,
+    MAX_CONCURRENT_STREAMS
 )
 from .stream import H2Stream
 from .utilities import validate_headers, guard_increment_window
@@ -260,7 +261,17 @@ class H2Connection(object):
         self.client_side = client_side
 
         # Objects that store settings, including defaults.
-        self.local_settings = Settings(client=client_side)
+        #
+        # We set the MAX_CONCURRENT_STREAMS value to 100 because its default is
+        # unbounded, and that's a dangerous default because it allows
+        # essentially unbounded resources to be allocated regardless of how
+        # they will be used. 100 should be suitable for the average
+        # application. This default obviously does not apply to the remote
+        # peer's settings: the remote peer controls them!
+        self.local_settings = Settings(
+            client=client_side,
+            initial_values={MAX_CONCURRENT_STREAMS: 100}
+        )
         self.remote_settings = Settings(client=not client_side)
 
         # The curent value of the connection flow control windows on the
@@ -499,8 +510,20 @@ class H2Connection(object):
         ``send_headers`` has *already* been called, this will send trailers
         instead.
 
-        In all situations it is a protocol error to call ``send_headers`` more
-        than twice.
+        When acting as a server, you may call ``send_headers`` any number of
+        times allowed by the following rules, in this order:
+
+        - zero or more times with ``(':status', '1XX')`` (where ``1XX`` is a
+          placeholder for any 100-level status code).
+        - once with any other status header.
+        - zero or one time for trailers.
+
+        That is, you are allowed to send as many informational responses as you
+        like, followed by one complete response and zero or one HTTP trailer
+        blocks.
+
+        Clients may send one or two header blocks: one request block, and
+        optionally one trailer block.
 
         .. warning:: In HTTP/2, it is mandatory that all the HTTP/2 special
             headers (that is, ones whose header keys begin with ``:``) appear
@@ -1259,6 +1282,8 @@ class H2Connection(object):
         new_event = ConnectionTerminated()
         new_event.error_code = frame.error_code
         new_event.last_stream_id = frame.last_stream_id
+        new_event.additional_data = (frame.additional_data
+                                     if frame.additional_data else None)
         events.append(new_event)
 
         return [], events
